package net.javajigi.user.model;

import static org.junit.Assert.assertEquals;
import net.javajigi.user.ExceedAdminUserException;
import net.javajigi.user.ExistedUserException;
import net.javajigi.user.dao.UserDao;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.Before;
import org.junit.Test;

public class UserTest {
	private Mockery context;
	private UserDao userDao;

	@Before
	public void setUp() throws Exception {
		context = new Mockery();
		userDao = context.mock(UserDao.class);
	}

	@Test
	public void createNormalUser() throws ExceedAdminUserException, ExistedUserException {
		final User normalUser = new NormalMockUser();

		context.checking(new Expectations() {{
				one(userDao).findUser(normalUser.getUserId());
				will(returnValue(null));
				one(userDao).insert(normalUser);
				never(userDao).countAdminUser();
			}});

		normalUser.setUserDao(userDao);
		normalUser.create();

		context.assertIsSatisfied();
	}

	@Test(expected = ExistedUserException.class)
	public void existedUserCreate() throws Exception {
		final User normalUser = new NormalMockUser();
		context.checking(new Expectations() {
			{
				one(userDao).findUser(normalUser.getUserId());
				will(returnValue(normalUser));
			}
		});

		normalUser.setUserDao(userDao);
		normalUser.create();

		context.assertIsSatisfied();
	}

	@Test
	public void exceedAdminWithnormaluser() throws Exception {
		User normalUser = new NormalMockUser();

		context.checking(new Expectations() {
			{
				never(userDao).countAdminUser();
			}
		});

		normalUser.setUserDao(userDao);
		assertEquals(true, normalUser.canCreate());
		
		context.assertIsSatisfied();
	}

	@Test
	public void exceedAdminWithadminUser() throws Exception {
		User adminUser = new AdminMockUser();

		context.checking(new Expectations() {{
				one(userDao).countAdminUser();
				will(returnValue(User.MAXIMUM_ADMIN_USER));
		}});

		adminUser.setUserDao(userDao);
		assertEquals(false, adminUser.canCreate());
		
		context.assertIsSatisfied();
	}
}
